--[[

SurfaceControl, controls what SurfaceLib returns from the Fetch method.
Copyright (C) 2007 rabbit.magtheridon@gmail.com

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

]]

SurfaceControl = AceLibrary("AceAddon-2.0"):new("AceEvent-2.0", "AceDB-2.0", "AceConsole-2.0")

SurfaceControl:RegisterDB("SurfaceControlDB")
SurfaceControl:RegisterDefaults("profile", {
	texture = nil,
})

SurfaceControl:RegisterChatCommand({ "/surfacecontrol" }, {
	type = "text",
	name = "Texture",
	desc = "Set the override texture for SurfaceLib.",
	usage = "<texture>",
	get = function() return SurfaceControl.db.profile.texture end,
	set = function(v)
		SurfaceControl.db.profile.texture = v
		SurfaceControl:UpdateAddons()
	end,
	validate = function(v)
		return AceLibrary("Surface-1.0"):IsValid(v)
	end,
}, "SURFACECONTROL")

local orig = nil
local oldRevision = nil

function SurfaceControl:OnEnable()
	if not AceLibrary:HasInstance("Surface-1.0") then
		self:Print("SurfaceControl requires Surface-1.0 to function properly.")
	end

	self:RegisterEvent("AceEvent_FullyInitialized")
	self:RegisterEvent("AceLibrary_Register")

	if AceLibrary("AceEvent-2.0"):IsFullyInitialized() then
		self:AceEvent_FullyInitialized()
	end
end

function SurfaceControl:AceEvent_FullyInitialized()
	self:HookFetch(AceLibrary("Surface-1.0"))
end

function SurfaceControl:AceLibrary_Register(major, instance)
	if major == "Surface-1.0" then
		self:HookFetch(instance)
	end
end

function SurfaceControl:HookFetch(instance)
	if type(instance.Fetch) ~= "function" then return end
	local _, rev = instance:GetLibraryVersion()
	if oldRevision and rev <= oldRevision then return end
	if not oldRevision then oldRevision = rev end
	orig = instance.Fetch
	instance.Fetch = SurfaceControl.FetchOverride

	self:UpdateAddons()
end

function SurfaceControl:UpdateAddons()
	-- Any addons that need to be reset
	if type(aUF) == "table" and type(aUF.CallUnitMethods) == "function" then
		aUF:CallUnitMethods("BarTexture")
	end

	if type(CBRipoff) == "table" and type(CBRipoff.Layout) == "function" then
		CBRipoff:Layout("CastingBar")
		CBRipoff:Layout("BREATH", "MirrorBar")
		CBRipoff:Layout("EXHAUSTION", "MirrorBar")
		CBRipoff:Layout("FEIGNDEATH", "MirrorBar")
	end
end

function SurfaceControl:FetchOverride(texture)
	if type(SurfaceControl.db.profile.texture) ~= "string" or not AceLibrary("Surface-1.0"):IsValid(texture) then
		return orig(self, texture)
	end
	return orig(self, SurfaceControl.db.profile.texture)
end

